searchData={"content_type":"text/markdown","items":[{"doc":"ProtoMock is a library for mocking Elixir protocols.","ref":"ProtoMock.html","title":"ProtoMock","type":"module"},{"doc":"ProtoMock was built to support using protocols, rather than behaviours or plain\nmodules, for modeling and accessing external APIs. When external APIs are modeled with\nprotocols, ProtoMock can provide mocking capabilities.\n\nModeling external APIs with protocols provides these benefits:\n\n* API transparency\n* IDE navigability\n* Compiler / dialyzer error detection\n* Flexible options for mocking (for example using custom fake objects for some tests, instead of a mocking library)\n\nIt is not expected that ProtoMock would be useful for more traditional protocol use\ncases, wherein protocols such as `Enumerable` provide a common interface for operating on\ndisparate data structures. In such situations, there is no value in testing with mocks,\ntherefore ProtoMock has no role.","ref":"ProtoMock.html#module-motivation-use-case","title":"Motivation / use case - ProtoMock","type":"module"},{"doc":"Add `protomock` to your list of dependencies in `mix.exs`:\n\n```elixir\n    def deps do\n      [\n        # ...\n        {:protomock, \"~> 1.0.0\", only: :test}\n      ]\n    end\n```\n\nBecause ProtoMock generates implementations of the protocols that it mocks, we need to\ndisable [protocol consolidation](https://hexdocs.pm/elixir/1.15.6/Protocol.html#module-consolidation) for the `:test` environment in `mix.exs`:\n\n```elixir\n    def project do\n      [\n        # ...\n        consolidate_protocols: Mix.env() != :test\n      ]\n    end\n```","ref":"ProtoMock.html#module-getting-started","title":"Getting started - ProtoMock","type":"module"},{"doc":"Following the traditional [Mox example](https://hexdocs.pm/mox/Mox.html#module-example),\nimagine that we have an app that displays the weather. To retrieve weather data,\nwe use an external weather API called AcmeWeather, and we model the API with our own\nprotocol:\n\n```elixir\n    defprotocol MyApp.WeatherAPI do\n      @type lat_long :: {float(), float()}\n      @type api_result :: {:ok, float()} | {:error, String.t()}\n\n      @spec temperature(t(), lat_long()) :: api_result()\n      def temperature(weather_api, lat_long)\n\n      @spec humidity(t(), lat_long()) :: api_result()\n      def humidity(weather_api, lat_long)\n    end\n```\n\nWe create a \"real\" implementation of `WeatherAPI` that calls out to the\nAcmeWeather API client:\n\n```elixir\n    defimpl MyApp.WeatherAPI, for: AcmeWeather.ApiConfig do\n      def temperature(api_config, {lat, long}) do\n        AcmeWeather.Client.get_temperature(lat, long, api_config)\n      end\n\n      def humidity(api_config, {lat, long}) do\n        AcmeWeather.Client.get_humidity(lat, long, api_config)\n      end\n    end\n```\n\nFor testing, however, we want to mock the service.\n\nContinuing with the [Mox example](https://hexdocs.pm/mox/Mox.html#module-example),\nimagine that our application code looks like:\n\n```elixir\n    defmodule MyApp.HumanizedWeather do\n      alias MyApp.WeatherAPI\n\n      def display_temp({lat, long}, weather_api) do\n        {:ok, temp} = WeatherAPI.temperature(weather_api, {lat, long})\n        \"Current temperature is #{temp} degrees\"\n      end\n\n      def display_humidity({lat, long}, weather_api) do\n        {:ok, humidity} = WeatherAPI.humidity(weather_api, {lat, long})\n        \"Current humidity is #{humidity}%\"\n      end\n    end\n```\n\nWe can test `HumanizedWeather` by mocking `WeatherAPI` with ProtoMock:\n\n```elixir\n    defmodule MyApp.HumanizedWeatherTest do\n      use ExUnit.Case, async: true\n\n      alias MyApp.HumanizedWeather\n      alias MyApp.WeatherAPI\n\n      test \"gets and formats temperature\" do\n        protomock =\n          ProtoMock.new(WeatherAPI)\n          |> ProtoMock.expect(&WeatherAPI.temperature/2, 1, fn _lat_long -> {:ok, 30} end)\n\n        assert HumanizedWeather.display_temp({50.06, 19.94}, protomock) ==\n                \"Current temperature is 30 degrees\"\n\n        ProtoMock.verify!(protomock)\n      end\n\n      test \"gets and formats humidity\" do\n        protomock =\n          ProtoMock.new(WeatherAPI)\n          |> ProtoMock.stub(&WeatherAPI.humidity/2, fn _lat_long -> {:ok, 60} end)\n\n        assert HumanizedWeather.display_humidity({50.06, 19.94}, protomock) ==\n              \"Current humidity is 60%\"\n      end\n    end\n```\nIn the first test, we use `expect/4` to declare that `WeatherAPI.temperature/2` should be called\nexactly once. The expectation is verified via `verify!/1`.\n\nIn the second test, we use `stub/3`, which does not set expectations on the number of times\nthe mocked function should be called, therefore we do not need to verify.","ref":"ProtoMock.html#module-example","title":"Example - ProtoMock","type":"module"},{"doc":"`new/2` can be used to create a `ProtoMock` instance that delegates to another implementation \nof the protocol of interest. By default, the `ProtoMock` instance will proxy any function \ninvocations to the delegate. Behavior can be overridden & customized on a per-function basis.","ref":"ProtoMock.html#module-delegation","title":"Delegation - ProtoMock","type":"module"},{"doc":"The `ProtoMock` module is a GenServer. Each time we create a `ProtoMock` with `new/1`,\nwe start a new `ProtoMock` GenServer that is linked to the calling process - typically\nan ExUnit test process. When the test pid dies, the `ProtoMock` GenServer dies with it.\n\n`expect/4` and `stub/3` modify the `ProtoMock` GenServer state to tell the `ProtoMock`\nhow it will be used and how it should respond. As the `ProtoMock` instance is used to\ndispatch functions of a mocked protocol, it records each function invocation.\n`verify!/1` compares the function invocations to the expectations defined via\n`expect/4`, and raises in case of an expectations mismatch.","ref":"ProtoMock.html#module-under-the-hood-a-genserver","title":"Under the hood: a GenServer - ProtoMock","type":"module"},{"doc":"In order to feel familiar to developers, the ProtoMock API was modeled after the [Mox API](https://hexdocs.pm/mox/Mox.html).\n\nSome differences worth noting:\n\n* ProtoMock has no concept of private mode or global mode. It's expected that each ExUnit\n  test will create its own instance or instances of `ProtoMock` that are implicitly private\n  to the test pid, thereby always being safe for `async: true`\n* Similarly, ProtoMock has no concept of allowances. Each `ProtoMock` instance is just a\n  GenServer that can be used freely and without worry by any process spawned by an\n  ExUnit test process (provided that the child process does not interact with other tests).\n* Rather than specificying expectations and stubs with a module name and a function name,\n  e.g. `(MyAPIModule, :my_api_function ...)`, ProtoMock uses function captures, e.g.\n  `&MyApiProtocol.my_api_function/2`. As a benefit, API mismatches between actual code and\n  expectations/stubs will be flagged by the compiler.\n* `stub_with` and `verify_on_exit` are not meaningful when using ProtoMock, and they\n  are not implemented.\n\n<!--","ref":"ProtoMock.html#module-comparison-to-mox-https-hexdocs-pm-mox-mox-html","title":"Comparison to [Mox](https://hexdocs.pm/mox/Mox.html) - ProtoMock","type":"module"},{"doc":"ProtoMock supports runtime type checking of mocked functions, via code adopted from [Hammox](https://hexdocs.pm/hammox/Hammox.html).\nType checking is disabled by default. It can be enabled via `enable_type_checking/0`.\nSee `enable_type_checking/0` for details on how type checking works. -->","ref":"ProtoMock.html#module-runtime-type-checking","title":"Runtime type checking - ProtoMock","type":"module"},{"doc":"ProtoMock supports the idea that each test should be its own little parallel universe,\nwithout any modifiable state shared between tests. It avoids practices common in mocking \nlibraries such as setting/resetting `Application` environment variables. Such practices \ncreate potential collisions between tests that must be avoided with `async: false`. \nProtoMock believes `async` should always be `true`!\n\nProtoMock aims to provide an easy-on-the-eyes, function-oriented API that doesn't rely on \nmacros and doesn't require wrapping test code in closures.\n\nIn alignment with José Valim's [\"mocks as nouns\"](https://blog.plataformatec.com.br/2015/10/mocks-and-explicit-contracts/) suggestion,\nProtoMock takes an approach to mocking that is centered on data structures. Good things happen\nwhen we customize behavior by providing custom data structures at the point of usage, without\nresorting to global variables such as the `Application` environment. \n\nWe improve our code by creating pathways to get our data structures where we need them. The same \npathways used for tests can also be used to customize production behavior. We can use the pathways\nto provide test fakes instead of mocks created by a mocking library, which is a much better way \nto mock in many situations. \n\nProtocol implementations are one form of such data structures. Libraries such as [Tesla](https://hexdocs.pm/tesla/Tesla.html) \nand [Req](https://hexdocs.pm/req/readme.html) provide built-in mocking capabilities via \napplication-specific data structures - Tesla through custom [adapters](https://hexdocs.pm/tesla/Tesla.Client.html#t:t/0) \nand Req through [fake request adapters](https://hexdocs.pm/req/Req.Request.html#module-adapter). \nWhen libraries provide such hooks, we might decide we wouldn't benefit from using a protocol to \nmodel/mock a given external service. What's important is that we mock using data structures \n(rather than modules or fancy runtime tricks), and use them locally (not globally). \n\nProtoMock is often spotted muttering *\"mock locally, with nouns\"* over and over again.","ref":"ProtoMock.html#module-goals-and-philosophy","title":"Goals and philosophy - ProtoMock","type":"module"},{"doc":"Expects `mocked_function` to be dispatched to `protomock` `invocation_count` times.\n\nWhen `mocked_function` is dispatched, the `impl` function will be invoked, using the\narguments passed to `mocked_function` (except for the first arg - see next paragraph).\nThe value returned from `impl` will be returned from `mocked_function`.\n\nThe `impl` function must have an arity that is one less than the arity of\n`mocked_function`. Because `mocked_function` is a protocol function, its first\nargument is the data structure that implements the protocol, which in this case is\n`protomock`. The `impl` function has no need for this data structure, so it is omitted\nfrom the `impl` argument list.\n\nWhen `expect/4` is invoked, any previously declared stubs for the same `mocked_function`\nwill be removed. This ensures that `expect` will fail if the function is called more\nthan `invocation_count` times. If `stub/3` is invoked after `expect/4` for the same\n`mocked_function`, the stub will be used after all expectations are fulfilled.\n\n`expect/4` will raise an `ArgumentError` if `mocked_function` is not a member function\nof the protocol mocked by `protomock`, as indicated via `new/1`.","ref":"ProtoMock.html#expect/4","title":"ProtoMock.expect/4","type":"function"},{"doc":"To expect `WeatherAPI.temperature/2` to be called once:\n\n    protomock =\n      ProtoMock.new(WeatherAPI)\n      |> ProtoMock.expect(&WeatherAPI.temperature/2, fn _lat_long -> {:ok, 30} end)\n\nTo expect `WeatherAPI.temperature/2` to be called five times:\n\n    protomock =\n      ProtoMock.new(WeatherAPI)\n      |> ProtoMock.expect(&WeatherAPI.temperature/2, 5, fn _lat_long -> {:ok, 30} end)\n\nTo expect `WeatherAPI.temperature/2` to not be called:\n\n    protomock =\n      ProtoMock.new(WeatherAPI)\n      |> ProtoMock.expect(&WeatherAPI.temperature/2, 0, fn _lat_long -> {:ok, 30} end)\n\n`expect/4` can be invoked multiple times for the same `mocked_function`, permitting\ndifferent behaviors for each invocation. For example, we could test that our code\nwill try an API call three times before giving up:\n\n    protomock =\n      ProtoMock.new(WeatherAPI)\n      |> ProtoMock.expect(&WeatherAPI.temperature/2, 2, fn _ -> {:error, :unreachable} end)\n      |> ProtoMock.expect(&WeatherAPI.temperature/2, 1, fn _ -> {:ok, 30} end)\n\n    lat_long = {0.0, 0.0}\n\n    log = capture_log(fn ->\n      humanized_temp = HumanizedWeather.display_temp(lat_long, protomock)\n      assert humanized_temp == \"It's currently 30 degrees\"\n    end)\n\n    assert log =~ \"attempt 1 failed\"\n    assert log =~ \"attempt 2 failed\"\n    assert log =~ \"attempt 3 succeeded\"\n\n    ProtoMock.expect(protomock, &WeatherAPI.temperature/2, 3, fn _ -> {:error, :unreachable} end)\n\n    result = HumanizedWeather.display_temp(lat_long, protomock)\n    assert result == \"Current temperature is unavailable\"","ref":"ProtoMock.html#expect/4-examples","title":"Examples - ProtoMock.expect/4","type":"function"},{"doc":"","ref":"ProtoMock.html#has_expectations?/2","title":"ProtoMock.has_expectations?/2","type":"function"},{"doc":"","ref":"ProtoMock.html#has_stubs?/2","title":"ProtoMock.has_stubs?/2","type":"function"},{"doc":"Creates a new instance of `ProtoMock` that mocks the given `protocol`.\n\nAfter creating a new `ProtoMock`, tests can add expectations and stubs to the instance\nusing `expect/4` and `stub/3`. With expectations and stubs in place, the `ProtoMock`\ninstance can be provided to the code under test, and used by the code under test\nwhere it expects an implementation of `protocol`.\n\nIf ProtoMock does not yet implement `protocol`, `new/1` will generate an implementation.\n\nSubsequent calls to `expect/4` and `stub/3` will verify that their mocked functions are\nmember functions of `protocol`.\n\nThe `ProtoMock` module is a GenServer. `new/1` starts a new instance of the GenServer\nthat is linked to the calling process, typically an ExUnit test pid. When the test pid\nexits, any child `ProtoMock` GenServers also exit.","ref":"ProtoMock.html#new/1","title":"ProtoMock.new/1","type":"function"},{"doc":"Creates a new instance of `ProtoMock` that mocks the given `protocol` and delegates\nnon-customized function calls to `delegate`.\n\n`delegate` must be a data structure that implements `protocol`. By default, the `ProtoMock`\ninstance returned by `new/2` will delegate any function calls to `delegate`. Delegation\ncan be \"overridden\" on a per-function basis by calling `expect/4` or `stub/3` for any\nfunction of interest.\n\nFor example, consider the following `Calculator` protocol:\n\n    defprotocol Calculator do\n      def add(calculator, x, y)\n\n      def mult(calculator, x, y)\n\n      def sqrt(calulator, x)\n    end\n\nAnd suppose we have an implentation called `RealCalculator` that implements the protocol\nfunctions in the expected manner. We can create a `ProtoMock` instance stubs out one\n`Calculator` function while delegating the others to an instance of `RealCalculator`:\n\n    protomock =\n      ProtoMock.new(Calculator, RealCalculator.new())\n      |> ProtoMock.stub(&Calculator.add/3, fn _calculator, _x, _y -> :overridden end)\n\n    Calculator.add(protomock, 1, 2)  # => :overridden\n    Calculator.mult(protomock, 1, 2) # => 2\n    Calculator.sqrt(protomock, 4)    # => 2.0\n\nWhen delegating, `expect/4` and `verify!/1` work together as they normally do - `verify!/1`\nwill raise if expectations are not met:\n\n    protomock =\n      ProtoMock.new(Calculator, real_calculator)\n      |> ProtoMock.expect(&Calculator.add/3, 1, fn _calculator, _x, _y -> :overridden end)\n\n    ProtoMock.verify!(protomock) # raises VerificationError\n\nWhen a function is overriden using `expect/4`, ProtoMock will raise an error if the\nfunction is called more times than expected, as in the non-delegated case. If instead\nit's desired that the function should be delegated after expectations are met, `stub/3`\ncan be used in conjunction with `expect/4` to establish the delegation:\n\n    real_calculator = RealCalculator.new()\n\n    protomock =\n      ProtoMock.new(Calculator, real_calculator)\n      |> ProtoMock.expect(&Calculator.add/3, 1, fn _calculator, _x, _y -> :overridden end)\n      |> ProtoMock.stub(&Calculator.add/3, fn _calculator, x, y ->\n        Calculator.add(real_calculator, x, y)\n      end)\n\n    Calculator.add(protomock, 1, 2)  # => :overridden\n    Calculator.add(protomock, 1, 2)  # => 3","ref":"ProtoMock.html#new/2","title":"ProtoMock.new/2","type":"function"},{"doc":"Allows `mocked_function` to be dispatched to `protomock` and proxied to `impl`.\n\nWhen `mocked_function` is dispatched, the `impl` function will be invoked, using the\narguments passed to `mocked_function` (except for the first arg - see next paragraph).\nThe value returned from `impl` will be returned from `mocked_function`.\n\nThe `impl` function must have an arity that is one less than the arity of\n`mocked_function`. Because `mocked_function` is a protocol function, its first\nargument is the data structure that implements the protocol, which in this case is\n`protomock`. The `impl` function has no need for this data structure, so it is omitted\nfrom the `impl` argument list.\n\nUnlike expectations, stubs are never verified.\n\nIf expectations and stubs are defined for the same `mocked_function`, the stub is\ninvoked only after all expectations are fulfilled.\n\n`stub/3` will raise an `ArgumentError` if `mocked_function` is not a member function\nof the protocol mocked by `protomock`, as indicated via `new/1`.","ref":"ProtoMock.html#stub/3","title":"ProtoMock.stub/3","type":"function"},{"doc":"To allow `WeatherAPI.temperature/2` to be dispatched to a `ProtoMock` instance any\nnumber of times:\n\n    protomock =\n      ProtoMock.new(WeatherAPI)\n      |> ProtoMock.stub(&WeatherAPI.temperature/2, fn _lat_long -> {:ok, 30} end)\n\n`stub/3` will overwrite any previous calls to `stub/3`.","ref":"ProtoMock.html#stub/3-example","title":"Example - ProtoMock.stub/3","type":"function"},{"doc":"Verifies that all expectations have been fulfilled. Raises a `VerificationError` if expectations\nhave not been fulfilled.","ref":"ProtoMock.html#verify!/1","title":"ProtoMock.verify!/1","type":"function"},{"doc":"","ref":"ProtoMock.html#t:t/0","title":"ProtoMock.t/0","type":"opaque"},{"doc":"Error raised when a protocol function is invoked more times than expected.","ref":"ProtoMock.UnexpectedCallError.html","title":"ProtoMock.UnexpectedCallError","type":"exception"},{"doc":"Error raised by `ProtoMock.verify!/1` when expectations set via `ProtoMock.expect/4`\nhave not been satisfied.","ref":"ProtoMock.VerificationError.html","title":"ProtoMock.VerificationError","type":"exception"},{"doc":"# ProtoMock\n\n \n \n\nProtoMock is a library for mocking Elixir protocols.","ref":"readme.html","title":"ProtoMock","type":"extras"},{"doc":"ProtoMock was built to support using protocols, rather than behaviours or plain\nmodules, for modeling and accessing external APIs. When external APIs are modeled with\nprotocols, ProtoMock can provide mocking capabilities.\n\nModeling external APIs with protocols provides these benefits:\n\n* API transparency\n* IDE navigability\n* Compiler / dialyzer error detection\n* Flexible options for mocking (for example using custom fake objects for some tests, instead of a mocking library)\n\nIt is not expected that ProtoMock would be useful for more traditional protocol use\ncases, wherein protocols such as `Enumerable` provide a common interface for operating on\ndisparate data structures. In such situations, there is no value in testing with mocks,\ntherefore ProtoMock has no role.","ref":"readme.html#motivation-use-case","title":"Motivation / use case - ProtoMock","type":"extras"},{"doc":"Add `protomock` to your list of dependencies in `mix.exs`:\n\n```elixir\n    def deps do\n      [\n        # ...\n        {:protomock, \"~> 1.0.0\", only: :test}\n      ]\n    end\n```\n\nBecause ProtoMock generates implementations of the protocols that it mocks, we need to\ndisable [protocol consolidation](https://hexdocs.pm/elixir/1.15.6/Protocol.html#module-consolidation) for the `:test` environment in `mix.exs`:\n\n```elixir\n    def project do\n      [\n        # ...\n        consolidate_protocols: Mix.env() != :test\n      ]\n    end\n```","ref":"readme.html#getting-started","title":"Getting started - ProtoMock","type":"extras"},{"doc":"Following the traditional [Mox example](https://hexdocs.pm/mox/Mox.html#module-example),\nimagine that we have an app that displays the weather. To retrieve weather data,\nwe use an external weather API called AcmeWeather, and we model the API with our own\nprotocol:\n\n```elixir\n    defprotocol MyApp.WeatherAPI do\n      @type lat_long :: {float(), float()}\n      @type api_result :: {:ok, float()} | {:error, String.t()}\n\n      @spec temperature(t(), lat_long()) :: api_result()\n      def temperature(weather_api, lat_long)\n\n      @spec humidity(t(), lat_long()) :: api_result()\n      def humidity(weather_api, lat_long)\n    end\n```\n\nWe create a \"real\" implementation of `WeatherAPI` that calls out to the\nAcmeWeather API client:\n\n```elixir\n    defimpl MyApp.WeatherAPI, for: AcmeWeather.ApiConfig do\n      def temperature(api_config, {lat, long}) do\n        AcmeWeather.Client.get_temperature(lat, long, api_config)\n      end\n\n      def humidity(api_config, {lat, long}) do\n        AcmeWeather.Client.get_humidity(lat, long, api_config)\n      end\n    end\n```\n\nFor testing, however, we want to mock the service.\n\nContinuing with the [Mox example](https://hexdocs.pm/mox/Mox.html#module-example),\nimagine that our application code looks like:\n\n```elixir\n    defmodule MyApp.HumanizedWeather do\n      alias MyApp.WeatherAPI\n\n      def display_temp({lat, long}, weather_api) do\n        {:ok, temp} = WeatherAPI.temperature(weather_api, {lat, long})\n        \"Current temperature is #{temp} degrees\"\n      end\n\n      def display_humidity({lat, long}, weather_api) do\n        {:ok, humidity} = WeatherAPI.humidity(weather_api, {lat, long})\n        \"Current humidity is #{humidity}%\"\n      end\n    end\n```\n\nWe can test `HumanizedWeather` by mocking `WeatherAPI` with ProtoMock:\n\n```elixir\n    defmodule MyApp.HumanizedWeatherTest do\n      use ExUnit.Case, async: true\n\n      alias MyApp.HumanizedWeather\n      alias MyApp.WeatherAPI\n\n      test \"gets and formats temperature\" do\n        protomock =\n          ProtoMock.new(WeatherAPI)\n          |> ProtoMock.expect(&WeatherAPI.temperature/2, 1, fn _lat_long -> {:ok, 30} end)\n\n        assert HumanizedWeather.display_temp({50.06, 19.94}, protomock) ==\n                \"Current temperature is 30 degrees\"\n\n        ProtoMock.verify!(protomock)\n      end\n\n      test \"gets and formats humidity\" do\n        protomock =\n          ProtoMock.new(WeatherAPI)\n          |> ProtoMock.stub(&WeatherAPI.humidity/2, fn _lat_long -> {:ok, 60} end)\n\n        assert HumanizedWeather.display_humidity({50.06, 19.94}, protomock) ==\n              \"Current humidity is 60%\"\n      end\n    end\n```\nIn the first test, we use `expect/4` to declare that `WeatherAPI.temperature/2` should be called\nexactly once. The expectation is verified via `verify!/1`.\n\nIn the second test, we use `stub/3`, which does not set expectations on the number of times\nthe mocked function should be called, therefore we do not need to verify.","ref":"readme.html#example","title":"Example - ProtoMock","type":"extras"},{"doc":"`new/2` can be used to create a `ProtoMock` instance that delegates to another implementation \nof the protocol of interest. By default, the `ProtoMock` instance will proxy any function \ninvocations to the delegate. Behavior can be overridden & customized on a per-function basis.","ref":"readme.html#delegation","title":"Delegation - ProtoMock","type":"extras"},{"doc":"The `ProtoMock` module is a GenServer. Each time we create a `ProtoMock` with `new/1`,\nwe start a new `ProtoMock` GenServer that is linked to the calling process - typically\nan ExUnit test process. When the test pid dies, the `ProtoMock` GenServer dies with it.\n\n`expect/4` and `stub/3` modify the `ProtoMock` GenServer state to tell the `ProtoMock`\nhow it will be used and how it should respond. As the `ProtoMock` instance is used to\ndispatch functions of a mocked protocol, it records each function invocation.\n`verify!/1` compares the function invocations to the expectations defined via\n`expect/4`, and raises in case of an expectations mismatch.","ref":"readme.html#under-the-hood-a-genserver","title":"Under the hood: a GenServer - ProtoMock","type":"extras"},{"doc":"In order to feel familiar to developers, the ProtoMock API was modeled after the [Mox API](https://hexdocs.pm/mox/Mox.html).\n\nSome differences worth noting:\n\n* ProtoMock has no concept of private mode or global mode. It's expected that each ExUnit\n  test will create its own instance or instances of `ProtoMock` that are implicitly private\n  to the test pid, thereby always being safe for `async: true`\n* Similarly, ProtoMock has no concept of allowances. Each `ProtoMock` instance is just a\n  GenServer that can be used freely and without worry by any process spawned by an\n  ExUnit test process (provided that the child process does not interact with other tests).\n* Rather than specificying expectations and stubs with a module name and a function name,\n  e.g. `(MyAPIModule, :my_api_function ...)`, ProtoMock uses function captures, e.g.\n  `&MyApiProtocol.my_api_function/2`. As a benefit, API mismatches between actual code and\n  expectations/stubs will be flagged by the compiler.\n* `stub_with` and `verify_on_exit` are not meaningful when using ProtoMock, and they\n  are not implemented.\n\n<!--","ref":"readme.html#comparison-to-mox-https-hexdocs-pm-mox-mox-html","title":"Comparison to [Mox](https://hexdocs.pm/mox/Mox.html) - ProtoMock","type":"extras"},{"doc":"ProtoMock supports runtime type checking of mocked functions, via code adopted from [Hammox](https://hexdocs.pm/hammox/Hammox.html).\nType checking is disabled by default. It can be enabled via `enable_type_checking/0`.\nSee `enable_type_checking/0` for details on how type checking works. -->","ref":"readme.html#runtime-type-checking","title":"Runtime type checking - ProtoMock","type":"extras"},{"doc":"ProtoMock supports the idea that each test should be its own little parallel universe,\nwithout any modifiable state shared between tests. It avoids practices common in mocking \nlibraries such as setting/resetting `Application` environment variables. Such practices \ncreate potential collisions between tests that must be avoided with `async: false`. \nProtoMock believes `async` should always be `true`!\n\nProtoMock aims to provide an easy-on-the-eyes, function-oriented API that doesn't rely on \nmacros and doesn't require wrapping test code in closures.\n\nIn alignment with José Valim's [\"mocks as nouns\"](https://blog.plataformatec.com.br/2015/10/mocks-and-explicit-contracts/) suggestion,\nProtoMock takes an approach to mocking that is centered on data structures. Good things happen\nwhen we customize behavior by providing custom data structures at the point of usage, without\nresorting to global variables such as the `Application` environment. \n\nWe improve our code by creating pathways to get our data structures where we need them. The same \npathways used for tests can also be used to customize production behavior. We can use the pathways\nto provide test fakes instead of mocks created by a mocking library, which is a much better way \nto mock in many situations. \n\nProtocol implementations are one form of such data structures. Libraries such as [Tesla](https://hexdocs.pm/tesla/Tesla.html) \nand [Req](https://hexdocs.pm/req/readme.html) provide built-in mocking capabilities via \napplication-specific data structures - Tesla through custom [adapters](https://hexdocs.pm/tesla/Tesla.Client.html#t:t/0) \nand Req through [fake request adapters](https://hexdocs.pm/req/Req.Request.html#module-adapter). \nWhen libraries provide such hooks, we might decide we wouldn't benefit from using a protocol to \nmodel/mock a given external service. What's important is that we mock using data structures \n(rather than modules or fancy runtime tricks), and use them locally (not globally). \n\nProtoMock is often spotted muttering *\"mock locally, with nouns\"* over and over again.","ref":"readme.html#goals-and-philosophy","title":"Goals and philosophy - ProtoMock","type":"extras"}]}